@@ -201,7 +201,7 @@ func main() {
 				},
 				cli.BoolTFlag{
 					Name:  "skip-needs",
-					Usage: `do not automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided. Defaults to true when --include-needs is not provided`,
+					Usage: `do not automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided. Defaults to true when --include-needs or --include-transitive-needs is not provided`,
 				},
 				cli.BoolFlag{
 					Name:  "include-needs",
@@ -286,6 +286,10 @@ func main() {
 					Name:  "include-needs",
 					Usage: `automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided`,
 				},
+				cli.BoolFlag{
+					Name:  "include-transitive-needs",
+					Usage: `like --include-needs, but also includes transitive needs (needs of needs). Does nothing when when --selector/-l flag is not provided. Overrides exclusions of other selectors and conditions.`,
+				},
 				cli.BoolFlag{
 					Name:  "skip-deps",
 					Usage: `skip running "helm repo update" and "helm dependency build"`,
@@ -414,12 +418,16 @@ func main() {
 				},
 				cli.BoolTFlag{
 					Name:  "skip-needs",
-					Usage: `do not automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided. Defaults to true when --include-needs is not provided`,
+					Usage: `do not automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided. Defaults to true when --include-needs or --include-transitive-needs is not provided`,
 				},
 				cli.BoolFlag{
 					Name:  "include-needs",
 					Usage: `automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided`,
 				},
+				cli.BoolFlag{
+					Name:  "include-transitive-needs",
+					Usage: `like --include-needs, but also includes transitive needs (needs of needs). Does nothing when when --selector/-l flag is not provided. Overrides exclusions of other selectors and conditions.`,
+				},
 				cli.BoolFlag{
 					Name:  "wait",
 					Usage: `Override helmDefaults.wait setting "helm upgrade --install --wait"`,
@@ -487,12 +495,16 @@ func main() {
 				},
 				cli.BoolTFlag{
 					Name:  "skip-needs",
-					Usage: `do not automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided. Defaults to true when --include-needs is not provided`,
+					Usage: `do not automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided. Defaults to true when --include-needs or --include-transitive-needs is not provided`,
 				},
 				cli.BoolFlag{
 					Name:  "include-needs",
 					Usage: `automatically include releases from the target release's "needs" when --selector/-l flag is provided. Does nothing when when --selector/-l flag is not provided`,
 				},
+				cli.BoolFlag{
+					Name:  "include-transitive-needs",
+					Usage: `like --include-needs, but also includes transitive needs (needs of needs). Does nothing when when --selector/-l flag is not provided. Overrides exclusions of other selectors and conditions.`,
+				},
 				cli.BoolFlag{
 					Name:  "skip-diff-on-install",
 					Usage: "Skips running helm-diff on releases being newly installed on this apply. Useful when the release manifests are too huge to be reviewed, or it's too time-consuming to diff at all",
@@ -782,7 +794,11 @@ func (c configImpl) SkipNeeds() bool {
 }
 
 func (c configImpl) IncludeNeeds() bool {
-	return c.c.Bool("include-needs")
+	return c.c.Bool("include-needs") || c.IncludeTransitiveNeeds()
+}
+
+func (c configImpl) IncludeTransitiveNeeds() bool {
+	return c.c.Bool("include-transitive-needs")
 }
 
 // DiffConfig
