@@ -1848,7 +1848,7 @@ generated: 2019-05-16T15:42:45.50486+09:00
 	})
 	fs.Cwd = basePath
 	state = injectFs(state, fs)
-	errs := state.UpdateDeps(helm)
+	errs := state.UpdateDeps(helm, false)
 
 	want := []string{"/example", "./example", generatedDir}
 	if !reflect.DeepEqual(helm.Charts, want) {
@@ -2141,7 +2141,7 @@ func TestHelmState_NoReleaseMatched(t *testing.T) {
 				RenderedValues: map[string]interface{}{},
 			}
 			state.Selectors = []string{tt.labels}
-			errs := state.FilterReleases()
+			errs := state.FilterReleases(false)
 			if (errs != nil) != tt.wantErr {
 				t.Errorf("ReleaseStatuses() for %s error = %v, wantErr %v", tt.name, errs, tt.wantErr)
 				return
